@@ -1,7 +1,7 @@
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2014 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2016 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -1457,9 +1457,9 @@ mail_notrun(cl_t * line, char context, struct tm *since)
 time_t
 check_lavg(time_t lim)
     /* run a job based on system load average if one should be run
-     * and return the time to sleep */
+     * and return the next time fcron should wake */
 {
-    time_t tts = 0;
+    time_t nwt = now;
     lavg_t *l = NULL;
 
 #ifdef NOLOADAVG
@@ -1470,8 +1470,8 @@ check_lavg(time_t lim)
     }
 
 
-    tts = time_to_sleep(lim);
-    return tts;
+    nwt = next_wake_time(lim);
+    return nwt;
 #else
 
     int i = 0;
@@ -1506,10 +1506,10 @@ check_lavg(time_t lim)
         }
 
     /* we do this set here as the nextexe of lavg line may change before */
-    tts = time_to_sleep(lim);
+    nwt = next_wake_time(lim);
 
     if (lavg_list->num_entries == 0)
-        return tts;
+        return nwt;
 
     if ((i = getloadavg(l_avg, 3)) != 3)
         debug("got only %d lavg values", i);
@@ -1549,35 +1549,34 @@ check_lavg(time_t lim)
     }
 
 
-    if (lavg_list->num_entries == 0)
-        return tts;
-    else
-        return (LAVG_SLEEP < tts) ? LAVG_SLEEP : tts;
+    if (lavg_list->num_entries == 0) {
+        return nwt;
+    }
+    else {
+        return tmin(now + LAVG_SLEEP, nwt);
+    }
 
 #endif                          /* def NOLOADAVG */
 
 }
 
 time_t
-time_to_sleep(time_t lim)
-  /* return the time to sleep until next task have to be executed. */
+next_wake_time(time_t lim)
+  /* return the earliest between lim (e.g. when to save to disk)
+   * and the next time fcron should run a job */
 {
-    /* we set tts to a big value, unless some problems can occurs
-     * with files without any line */
-    time_t tts = lim;
-    time_t ti = time(NULL);
+    time_t nwt = lim;
 
     /* note : jobs in queue_base are sorted */
     if (queue_base != NULL) {
         if (queue_base->j_line->cl_nextexe < lim)
-            tts = queue_base->j_line->cl_nextexe;
+            nwt = queue_base->j_line->cl_nextexe;
     }
 
-    tts = tts - ti;
-    if (tts < 0)
-        tts = 0;
+    if (nwt < now)
+        nwt = now;
 
 /*      debug("Time to sleep: %lds", tts); */
 
-    return tts;
+    return nwt;
 }
